@@ -1,4 +1,4 @@
-/* streamripper.c -
+/* streamripper.c
  * This little app should be seen as a demo for how to use the stremripper lib. 
  * The only file you need from the /lib dir is rip_mananger.h, and perhapes 
  * util.h (for stuff like formating the number of bytes).
@@ -250,7 +250,7 @@ void print_usage()
     fprintf(stderr, "      -l seconds     - Number of seconds to run, otherwise runs forever\n");
     fprintf(stderr, "      -M megabytes   - Stop ripping after this many megabytes\n");
     fprintf(stderr, "      -q [start]     - Add sequence number to output file\n");
-    fprintf(stderr, "      -i             - Don't add ID3V1 Tags to output file\n");
+    fprintf(stderr, "      -i             - Don't add ID3 tags to output file\n");
     fprintf(stderr, "      -u useragent   - Use a different UserAgent than \"Streamripper\"\n");
     fprintf(stderr, "      -w rulefile    - Parse metadata using rules in file.\n");
     fprintf(stderr, "      -m timeout     - Number of seconds before force-closing stalled conn\n");
@@ -259,6 +259,7 @@ void print_usage()
     fprintf(stderr, "      -I interface   - Rip from specified interface (e.g. eth0)\n");
 #endif
     fprintf(stderr, "      -T             - Truncate duplicated tracks in incomplete\n");
+    fprintf(stderr, "      -E command     - Run external command to fetch metadata\n");
     fprintf(stderr, "      --quiet        - Don't print ripping status to console\n");
     fprintf(stderr, "      --debug        - Save debugging trace\n");
     fprintf(stderr, "      --xs_???       - Invoke splitpoint detection rules (see README/man page)\n");
@@ -291,7 +292,7 @@ void parse_arguments(int argc, char **argv)
 	if (argv[i][0] != '-')
 	    continue;
 
-	c = strchr("dDfIklLmMopRuw", argv[i][1]);
+	c = strchr("dDEfIklLmMopRuw", argv[i][1]);
         if (c != NULL) {
             if ((i == (argc-1)) || (argv[i+1][0] == '-')) {
 		fprintf(stderr, "option %s requires an argument\n", argv[i]);
@@ -303,11 +304,14 @@ void parse_arguments(int argc, char **argv)
 	case 'a':
 	    /* Create single file output + cue sheet */
 	    m_opt.flags |= OPT_SINGLE_FILE_OUTPUT;
-	    m_opt.output_file[0] = 0;
+	    m_opt.showfile_pattern[0] = 0;
 	    if (i == (argc-1) || argv[i+1][0] == '-')
 		break;
 	    i++;
-	    strncpy (m_opt.output_file, argv[i], SR_MAX_PATH);
+	    strncpy (m_opt.showfile_pattern, argv[i], SR_MAX_PATH);
+	    break;
+	case 'A':
+	    m_opt.flags ^= OPT_INDIVIDUAL_TRACKS;
 	    break;
 	case 'c':
 	    m_opt.flags ^= OPT_AUTO_RECONNECT;
@@ -320,6 +324,11 @@ void parse_arguments(int argc, char **argv)
 	    i++;
 	    strncpy(m_opt.output_pattern, argv[i], SR_MAX_PATH);
 	    break;
+	case 'E':
+	    m_opt.flags |= OPT_EXTERNAL_CMD;
+	    i++;
+	    strncpy(m_opt.ext_cmd, argv[i], SR_MAX_PATH);
+	    break;
 	case 'f':
 	    i++;
 	    printf ("Error: -f dropstring option is obsolete. "
@@ -332,6 +341,10 @@ void parse_arguments(int argc, char **argv)
 	case 'i':
 	    m_opt.flags ^= OPT_ADD_ID3;
 	    break;
+	case 'I':
+	    i++;
+	    strncpy(m_opt.if_name, argv[i], SR_MAX_PATH);
+	    break;
 	case 'k':
 	    i++;
 	    m_opt.dropcount = atoi(argv[i]);
@@ -341,10 +354,19 @@ void parse_arguments(int argc, char **argv)
 	    time(&m_stop_time);
 	    m_stop_time += atoi(argv[i]);
 	    break;
+	case 'L':
+	    i++;
+	    strncpy(m_opt.pls_file, argv[i], SR_MAX_PATH);
+	    break;
 	case 'm':
 	    i++;
 	    m_opt.timeout = atoi(argv[i]);
 	    break;
+ 	case 'M':
+ 	    i++;
+ 	    m_opt.maxMB_rip_size = atoi(argv[i]);
+ 	    m_opt.flags |= OPT_CHECK_MAX_BYTES;
+ 	    break;
 	case 'o':
 	    i++;
 	    m_opt.overwrite = string_to_overwrite_opt (argv[i]);
@@ -376,9 +398,7 @@ void parse_arguments(int argc, char **argv)
 	    if (i == (argc-1) || argv[i+1][0] == '-')
 		break;
 	    i++;
-
 	    c = strstr(argv[i], ":");
-
 	    if (NULL == c) {
 	    	m_opt.relay_port = atoi(argv[i]);
 	    } else {
@@ -386,7 +406,10 @@ void parse_arguments(int argc, char **argv)
 		strncpy(m_opt.relay_ip, argv[i], SR_MAX_PATH);
 		m_opt.relay_port = atoi(++c);
  	    }
-
+	    break;
+	case 'R':
+	    i++;
+	    m_opt.max_connections = atoi(argv[i]);
 	    break;
 	case 's':
 	    m_opt.flags ^= OPT_SEPERATE_DIRS;
@@ -402,7 +425,7 @@ void parse_arguments(int argc, char **argv)
 	    strncpy(m_opt.useragent, argv[i], MAX_USERAGENT_STR);
 	    break;
 	case 'v':
-	    printf("Streamripper %s by Jon Clegg <jonclegg@yahoo.com>\n", SRVERSION);
+	    printf("Streamripper %s\n", SRVERSION);
 	    exit(0);
 	case 'w':
 	    i++;
@@ -412,26 +435,6 @@ void parse_arguments(int argc, char **argv)
 	    m_opt.flags ^= OPT_SEARCH_PORTS;
 	    m_opt.max_port = m_opt.relay_port+1000;
 	    break;
-	case 'A':
-	    m_opt.flags ^= OPT_INDIVIDUAL_TRACKS;
-	    break;
-	case 'I':
-	    i++;
-	    strncpy(m_opt.if_name, argv[i], SR_MAX_PATH);
-	    break;
-	case 'L':
-	    i++;
-	    strncpy(m_opt.pls_file, argv[i], SR_MAX_PATH);
-	    break;
- 	case 'M':
- 	    i++;
- 	    m_opt.maxMB_rip_size = atoi(argv[i]);
- 	    m_opt.flags |= OPT_CHECK_MAX_BYTES;
- 	    break;
-	case 'R':
-	    i++;
-	    m_opt.max_connections = atoi(argv[i]);
-	    break;
 	case '-':
 	    parse_extended_options(&argv[i][2]);
 	    break;
