@@ -68,9 +68,9 @@ filelib_init (BOOL do_individual_tracks,
 	      BOOL keep_incomplete,
 	      BOOL do_show_file,
 	      int content_type,
-	      char* show_file_name,
 	      char* output_directory,
 	      char* output_pattern,
+	      char* showfile_pattern,
 	      int get_separate_dirs,
 	      int get_date_stamp,
 	      char* icy_name);
