@@ -1,5 +1,4 @@
-/* rip_manager.c - jonclegg@yahoo.com
- * ties together lots of shit, main entry point for clients to call
+/* rip_manager.c
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -44,8 +43,6 @@
 #include "socklib.h"
 #include "http.h"
 #include "util.h"
-#include "riplive365.h"
-#include "ripshout.h"
 #include "findsep.h"
 #include "inet.h"
 #include "relaylib.h"
@@ -68,7 +65,6 @@ u_short			rip_mananger_get_relay_port();
  * Private functions
  ******************************************************************************/
 static void			ripthread(void *bla);
-static int			myrecv(char* buffer, int size);
 static error_code		start_relay(int content_type);
 static void			post_status(int status);
 //static error_code		start_track(char *track);
@@ -81,21 +77,20 @@ static void			destroy_subsystems();
 /******************************************************************************
  * Private Vars
  ******************************************************************************/
-static SR_HTTP_HEADER		m_info;
+static SR_HTTP_HEADER		m_http_info;
 static HSOCKET			m_sock;
+#if defined (commentout)
 static void			(*m_destroy_func)();
+#endif
 static RIP_MANAGER_INFO		m_ripinfo;		// used for UPDATE callback messages
 static RIP_MANAGER_OPTIONS	m_options;		// local copy of the options passed to rip_manager_start()
 static THREAD_HANDLE		m_hthread;		// rip thread handle
-static IO_DATA_INPUT		m_in;			// Gets raw stream data
-static IO_GET_STREAM		m_ripin;		// Raw stream data + song information
-//static IO_PUT_STREAM		m_ripout;		// Generic output interface
 static void			(*m_status_callback)(int message, void *data);
 static BOOL			m_ripping = FALSE;
 static u_long			m_bytes_ripped;
 static BOOL			m_write_data = TRUE;	// Should we actually write to a file or not
-static HSEM			m_started_sem;	// to prevent deadlocks when ripping is stopped before its
-													// started.
+static HSEM			m_started_sem;	// to prevent deadlocks when ripping is stopped before its started.
+
 static char* overwrite_opt_strings[] = {
     "",		// UNKNOWN
     "always",
@@ -174,6 +169,8 @@ void init_error_strings()
     SET_ERR_STR("SR_ERROR_PROGRAM_ERROR", 0x3a);
     SET_ERR_STR("SR_ERROR_TIMEOUT", 0x3b);
     SET_ERR_STR("SR_ERROR_SELECT_FAILED", 0x3c);
+    SET_ERR_STR("SR_ERROR_RESERVED_WINDOW_EMPTY", 0x3d);
+    SET_ERR_STR("SR_ERROR_CANT_BIND_ON_INTERFACE", 0x3e);
 }
 
 char*
@@ -209,21 +206,6 @@ post_status(int status)
 }
 
 
-int
-myrecv(char* buffer, int size)
-{
-    int ret;
-    /* GCS: Jun 5, 2004.  Here is where I think we are getting aussie's 
-       problem with the SR_ERROR_INVALID_METADATA or SR_ERROR_NO_TRACK_INFO
-       messages */
-    ret = socklib_recvall(&m_sock, buffer, size, m_options.timeout);
-    if (ret >= 0 && ret != size) {
-	debug_printf ("rip_manager_recv: expected %d, got %d\n",size,ret);
-	ret = SR_ERROR_RECV_FAILED;
-    }
-    return ret;
-}
-
 /* 
  * This is called by ripstream when we get a new track. 
  * most logic is handled by filelib_start() so we just
@@ -300,7 +282,9 @@ rip_manager_put_data(char *buf, int size)
 error_code
 rip_manager_put_raw_data(char *buf, int size)
 {
+#if defined (commentout)
     relaylib_send(buf, size, 1, 0);
+#endif
     filelib_write_show (buf, size);
     return SR_SUCCESS;
 }
@@ -326,7 +310,7 @@ start_relay(int content_type)
 char *
 client_relay_header_generate (int icy_meta_support)
 {
-    SR_HTTP_HEADER info = m_info;
+    SR_HTTP_HEADER info = m_http_info;
     char temp_icyname[MAX_SERVER_LEN];
     int ret;
     
@@ -335,7 +319,7 @@ client_relay_header_generate (int icy_meta_support)
     strcpy(info.icy_name, temp_icyname);
     
     headbuf = (char *) malloc(MAX_HEADER_LEN);
-    ret = httplib_construct_sc_response(&m_info, headbuf, MAX_HEADER_LEN, icy_meta_support);
+    ret = httplib_construct_sc_response(&m_http_info, headbuf, MAX_HEADER_LEN, icy_meta_support);
     if (ret != SR_SUCCESS) {
 	headbuf[0] = 0;
     }
@@ -355,18 +339,18 @@ ripthread(void *notused)
     error_code ret;
 
     if ((ret = start_ripping()) != SR_SUCCESS) {
-	threadlib_signel_sem(&m_started_sem);
+	threadlib_signal_sem(&m_started_sem);
 	post_error(ret);
 	goto DONE;
     }
     m_status_callback(RM_STARTED, (void *)NULL);
     post_status(RM_STATUS_BUFFERING);
-    threadlib_signel_sem(&m_started_sem);
+    threadlib_signal_sem(&m_started_sem);
 
     while(TRUE) {
         ret = ripstream_rip();
 
-	/* If the user told use to stop, well, then we bail */
+	/* If the user told us to stop, well, then we bail */
 	if (!m_ripping)
 	    break;
 
@@ -379,7 +363,8 @@ ripthread(void *notused)
 	 * the interface it's a weird combonation of threads
 	 * wnd locks, etc.. all i know is that this appeared to work
 	 */
-	if (m_bytes_ripped >= (m_options.maxMB_rip_size*1000000) &&
+        /* GCS Aug 23, 2003: m_bytes_ripped can still overflow */
+	if (m_bytes_ripped/1000000 >= (m_options.maxMB_rip_size) &&
 	    GET_CHECK_MAX_BYTES(m_options.flags)) {
 	    socklib_close(&m_sock);
 	    destroy_subsystems();
@@ -423,9 +408,10 @@ ripthread(void *notused)
 		// GCS testing...
 		filelib_shutdown();
 		ripstream_destroy();
+#if defined (commentout)
 		if (m_destroy_func)
 		    m_destroy_func();
-
+#endif
 		ret = start_ripping();
 		if (ret == SR_SUCCESS)
 		    break;
@@ -464,28 +450,38 @@ rip_manager_stop()
 	    return;
     
     // Make sure the ripping started before we try to stop
+    debug_printf ("Waiting for m_started_sem...\n");
     threadlib_waitfor_sem(&m_started_sem);
     m_ripping = FALSE;
 
-    /* Destroy the parsing rules */
-    /* destroy_metadata_parser (); */
-
     // Causes the code running in the thread to bail
+    debug_printf ("Closing m_sock...\n");
     socklib_close(&m_sock);
 
+    // Kill external process
+    if (m_ripinfo.ep) {
+	close_external (&m_ripinfo.ep);
+    }
+
     // blocks until everything is ok and closed
+    debug_printf ("Waiting for m_hthread to close...\n");
     threadlib_waitforclose(&m_hthread);
+    debug_printf ("Destroying subsystems...\n");
     destroy_subsystems();
+    debug_printf ("Destroying m_started_sem\n");
     threadlib_destroy_sem(&m_started_sem);
+    debug_printf ("Done with rip_manager_stop\n");
 }
 
 void
 destroy_subsystems()
 {
     ripstream_destroy();
+#if defined (commentout)
     if (m_destroy_func) {
 	m_destroy_func();
     }
+#endif
     relaylib_shutdown();
     socklib_cleanup();
     filelib_shutdown();
@@ -522,10 +518,17 @@ start_ripping()
     error_code ret;
     char *pproxy = m_options.proxyurl[0] ? m_options.proxyurl : NULL;
 
-    /*
-     * Connect to the stream
-     */
-    ret = inet_sc_connect(&m_sock, m_options.url, pproxy, &m_info, 
+    /* If proxy URL not spec'd on command line (or plugin field), 
+       check the environment variable */
+    if (!pproxy) {
+	char const *env_http_proxy = getenv ("http_proxy");
+	if (env_http_proxy) {
+	    strncpy(m_options.proxyurl, env_http_proxy, MAX_URL_LEN);
+	}
+    }
+
+    /* Connect to the stream */
+    ret = inet_sc_connect(&m_sock, m_options.url, pproxy, &m_http_info, 
 			  m_options.useragent, m_options.if_name);
     if (ret != SR_SUCCESS) {
 	goto RETURN_ERR;
@@ -533,7 +536,7 @@ start_ripping()
 
 #if defined (COMMENTOUT_FOR_OGG)
     /* GCS 09/10/04 - I wonder if this is worth doing */
-    if (!m_info.have_icy_name) {
+    if (!m_http_info.have_icy_name) {
 	ret = SR_ERROR_NOT_SHOUTCAST_STREAM;
 	goto RETURN_ERR;
     }
@@ -541,8 +544,8 @@ start_ripping()
 
     /* If the icy_name exists, but is empty, set to a bogus name so 
        that we can create the directory correctly, etc. */
-    if (strlen(m_info.icy_name) == 0) {
-	strcpy (m_info.icy_name, "Streamripper_rips");
+    if (strlen(m_http_info.icy_name) == 0) {
+	strcpy (m_http_info.icy_name, "Streamripper_rips");
     }
 
     /*
@@ -551,10 +554,10 @@ start_ripping()
      * bitrate etc.. 
      */
     memset(&m_ripinfo, 0, sizeof(RIP_MANAGER_INFO));
-    m_ripinfo.meta_interval = m_info.meta_interval;
-    m_ripinfo.bitrate = m_info.icy_bitrate;
-    strcpy(m_ripinfo.streamname, m_info.icy_name);
-    strcpy(m_ripinfo.server_name, m_info.server);
+    m_ripinfo.meta_interval = m_http_info.meta_interval;
+    m_ripinfo.bitrate = m_http_info.icy_bitrate;
+    strcpy(m_ripinfo.streamname, m_http_info.icy_name);
+    strcpy(m_ripinfo.server_name, m_http_info.server);
 
     /* Initialize file writing code. */
     ret = filelib_init
@@ -563,13 +566,13 @@ start_ripping()
 	     m_options.count_start,
 	     GET_KEEP_INCOMPLETE(m_options.flags),
 	     GET_SINGLE_FILE_OUTPUT(m_options.flags),
-	     m_info.content_type, 
-	     m_options.output_file,
+	     m_http_info.content_type, 
 	     m_options.output_directory,
 	     m_options.output_pattern,
+	     m_options.showfile_pattern,
 	     GET_SEPERATE_DIRS(m_options.flags),
 	     GET_DATE_STAMP(m_options.flags),
-	     m_info.icy_name);
+	     m_http_info.icy_name);
     if (ret != SR_SUCCESS)
 	goto RETURN_ERR;
 
@@ -578,25 +581,33 @@ start_ripping()
     m_status_callback(RM_OUTPUT_DIR, (void*)filelib_get_output_directory);
 #endif
 
-    /* prepares the ripshout lib for ripping */
-    ret = ripshout_init(&m_in, &m_ripin, m_info.meta_interval);
-    if (ret != SR_SUCCESS)
-	goto RETURN_ERR;
-    m_destroy_func = ripshout_destroy;
+    /* Start up external program to get metadata. */
+    m_ripinfo.ep = 0;
+    if (GET_EXTERNAL_CMD(m_options.flags)) {
+	debug_printf ("Spawn external: %s\n", m_options.ext_cmd);
+	m_ripinfo.ep = spawn_external (m_options.ext_cmd);
+	if (m_ripinfo.ep) {
+	    debug_printf ("Spawn external succeeded\n");
+	} else {
+	    debug_printf ("Spawn external failed\n");
+	}
+    }
 
-    /*
-     * ripstream is good to go, it knows how to get data, and where
+    /* ripstream is good to go, it knows how to get data, and where
      * it's sending it to
      */
     ripstream_destroy();
-    ret = ripstream_init(&m_ripin, 
-			 m_info.icy_name,
+    ret = ripstream_init(m_sock, 
+			 GET_MAKE_RELAY(m_options.flags),
+			 m_options.timeout, 
+			 m_http_info.icy_name,
 			 m_options.dropcount,
 			 &m_options.sp_opt,
 			 m_ripinfo.bitrate, 
-			 m_info.meta_interval,
-			 m_info.content_type, 
-			 GET_ADD_ID3(m_options.flags));
+			 m_http_info.meta_interval,
+			 m_http_info.content_type, 
+			 GET_ADD_ID3(m_options.flags),
+			 m_ripinfo.ep);
     if (ret != SR_SUCCESS) {
 	ripstream_destroy();
 	goto RETURN_ERR;
@@ -614,13 +625,14 @@ start_ripping()
 	ret = relaylib_init(GET_SEARCH_PORTS(m_options.flags), 
 			    m_options.relay_port, m_options.max_port, 
 			    &new_port, m_options.if_name, 
-			    m_options.max_connections, m_options.relay_ip);
+			    m_options.max_connections, m_options.relay_ip,
+			    m_http_info.meta_interval != NO_META_INTERVAL);
 	if (ret != SR_SUCCESS) {
 		goto RETURN_ERR;
 	}
 
 	m_options.relay_port = new_port;
-	start_relay(m_info.content_type);
+	start_relay(m_http_info.content_type);
 
 	if (0 != m_options.pls_file[0]) {
 		create_pls_file(new_port);
@@ -631,8 +643,10 @@ start_ripping()
 
 RETURN_ERR:
     socklib_close(&m_sock);
+#if defined (commentout)
     if (m_destroy_func)
 	m_destroy_func();
+#endif
     return ret;	
 }
 
@@ -655,9 +669,11 @@ rip_manager_start(void (*status_callback)(int message, void *data),
     socklib_init();
 
     init_error_strings();
-    m_in.get_input_data = myrecv;
     m_status_callback = status_callback;
+#if defined (commentout)
+    m_in.get_input_data = myrecv;
     m_destroy_func = NULL;
+#endif
     m_bytes_ripped = 0;
 
     /* Initialize the parsing rules */
@@ -716,10 +732,12 @@ set_rip_manager_options_defaults (RIP_MANAGER_OPTIONS *m_opt)
     m_opt->pls_file[0] = 0;
     m_opt->proxyurl[0] = 0;
     m_opt->url[0] = 0;
-    m_opt->output_file[0] = 0;
+    m_opt->showfile_pattern[0] = 0;
     m_opt->rules_file[0] = 0;
     strcpy(m_opt->useragent, "sr-POSIX/" SRVERSION);
-    m_opt->overwrite = OVERWRITE_ALWAYS;
+    m_opt->overwrite = OVERWRITE_LARGER;
+    m_opt->dropcount = 0;
+    m_opt->ext_cmd[0] = 0;
 
     // Defaults for splitpoint
     // Times are in ms
