@@ -1,4 +1,4 @@
-/* ripstream.c - jonclegg@yahoo.com
+/* ripstream.c
  * buffer stream data, when a track changes decodes the audio and 
  * finds a silent point to split the track
  *
@@ -34,15 +34,12 @@
 #include "debug.h"
 #include "filelib.h"
 #include "relaylib.h"
+#include "socklib.h"
+#include "external.h"
 
-/*********************************************************************************
- * Public functions
- *********************************************************************************/
-/* -- See ripstream.h -- */
-
-/*********************************************************************************
+/*****************************************************************************
  * Private functions
- *********************************************************************************/
+ *****************************************************************************/
 static error_code find_sep (u_long *pos1, u_long *pos2);
 static error_code end_track(u_long pos1, u_long pos2, TRACK_INFO* ti);
 static error_code start_track (TRACK_INFO* ti);
@@ -52,12 +49,17 @@ static void compute_cbuf2_size (SPLITPOINT_OPTIONS *sp_opt,
 static int ms_to_bytes (int ms, int bitrate);
 static int bytes_to_secs (unsigned int bytes);
 static void clear_track_info (TRACK_INFO* ti);
+static int ripstream_recvall (char* buffer, int size);
 
-/*********************************************************************************
+/* From ripshout */
+static error_code get_track_from_metadata (int size, char *newtrack);
+static error_code get_stream_data(char *data_buf, char *track_buf);
+
+/*****************************************************************************
  * Private Vars
- *********************************************************************************/
-static CBUF2			m_cbuffer;
-static IO_GET_STREAM	*m_in;
+ *****************************************************************************/
+#define DEFAULT_BUFFER_SIZE	1024
+
 static TRACK_INFO m_old_track;	    /* The track that's being ripped now */
 static TRACK_INFO m_new_track;	    /* The track that's gonna start soon */
 static TRACK_INFO m_current_track;  /* The metadata as I'm parsing it */
@@ -69,23 +71,27 @@ static BOOL			m_addID3tag = TRUE;
 static SPLITPOINT_OPTIONS	*m_sp_opt;
 static int			m_bitrate;
 static int			m_http_bitrate;
-static int			m_ripchunk_size;
 static int			m_meta_interval;
 static unsigned int		m_cue_sheet_bytes = 0;
+static External_Process*        m_external_process = 0;
 
-static int			m_cbuf2_size;
-static int			m_mi_to_cbuf2_start;
-static int			m_mi_to_cbuf2_end;
-static int			m_sw_start_to_cbuf2_end;
-static int			m_sw_end_to_cbuf2_end;
-static int			m_min_search_win;
-static int			m_drop_count;
-static int			m_track_count = 0;
+static int m_cbuf2_size;             /* blocks */
+static int m_rw_start_to_cb_end;     /* bytes */
+static int m_rw_start_to_sw_start;   /* milliseconds */
+static int m_rw_end_to_cb_end;       /* bytes */
+static int m_mic_to_cb_end;          /* blocks */
+
+static int m_drop_count;
+static int m_track_count = 0;
 static int m_content_type;
+static HSOCKET m_sock;
+static int m_have_relay;
+static int m_timeout;
+
+static int m_meta_interval;
+static int m_buffersize;
+static int m_chunkcount;
 
-/*
- * oddsock's id3 tags
- */
 typedef struct ID3V1st
 {
         char    tag[3];
@@ -111,22 +117,27 @@ typedef struct ID3V2framest {
 } ID3V2frame;
 
 
-
 error_code
-ripstream_init (IO_GET_STREAM *in, char *no_meta_name, 
+ripstream_init (HSOCKET sock, 
+		int have_relay,
+		int timeout, 
+		char *no_meta_name, 
 		int drop_count,
 		SPLITPOINT_OPTIONS *sp_opt, 
 		int bitrate, 
 		int meta_interval, 
 		int content_type, 
-		BOOL addID3tag)
+		BOOL addID3tag,
+		External_Process* ep)
 {
-    if (!in || !sp_opt || !no_meta_name) {
+    if (!sp_opt || !no_meta_name) {
 	printf ("Error: invalid ripstream parameters\n");
 	return SR_ERROR_INVALID_PARAM;
     }
 
-    m_in = in;
+    m_sock = sock;
+    m_have_relay = have_relay;
+    m_timeout = timeout;
     m_sp_opt = sp_opt;
     m_track_count = 0;
     m_addID3tag = addID3tag;
@@ -137,16 +148,22 @@ ripstream_init (IO_GET_STREAM *in, char *no_meta_name,
     m_content_type = content_type;
     /* GCS RMK: Ripchunk_size is the metaint size, or default size
        if stream doesn't have meta data */
-    m_ripchunk_size = in->getsize;
     m_meta_interval = meta_interval;
     m_cue_sheet_bytes = 0;
+    m_external_process = ep;
+
+    /* From ripshout */
+    m_buffersize = (m_meta_interval == NO_META_INTERVAL) 
+	    ? DEFAULT_BUFFER_SIZE : m_meta_interval;
+
+    m_chunkcount = 0;
 
     clear_track_info (&m_old_track);
     clear_track_info (&m_new_track);
     clear_track_info (&m_current_track);
     m_first_time_through = 1;
 
-    if ((m_getbuffer = malloc(in->getsize)) == NULL)
+    if ((m_getbuffer = malloc(m_buffersize)) == NULL)
 	return SR_ERROR_CANT_ALLOC_MEMORY;
 
     return SR_SUCCESS;
@@ -158,9 +175,8 @@ ripstream_destroy()
     debug_printf ("RIPSTREAM_DESTROY\n");
     if (m_getbuffer) {free(m_getbuffer); m_getbuffer = NULL;}
     m_find_silence = -1;
-    m_in = NULL;
     m_cbuf2_size = 0;
-    cbuf2_destroy(&m_cbuffer);
+    cbuf2_destroy (&g_cbuf2);
 
     clear_track_info (&m_old_track);
     clear_track_info (&m_new_track);
@@ -170,11 +186,11 @@ ripstream_destroy()
     m_no_meta_name[0] = '\0';
     m_track_count = 0;
     m_addID3tag = TRUE;
-}
 
-BOOL is_buffer_full()
-{
-    return (cbuf2_get_free(&m_cbuffer) - m_in->getsize) < m_in->getsize;
+    /* from ripshout */
+    m_buffersize = 0;
+    m_meta_interval = 0;
+    m_chunkcount = 0;
 }
 
 BOOL
@@ -215,6 +231,7 @@ clear_track_info (TRACK_INFO* ti)
     ti->artist[0] = 0;
     ti->title[0] = 0;
     ti->album[0] = 0;
+    ti->composed_metadata[0] = 0;
     ti->save_track = TRUE;
 }
 
@@ -226,9 +243,11 @@ copy_track_info (TRACK_INFO* dest, TRACK_INFO* src)
     strcpy (dest->artist, src->artist);
     strcpy (dest->title, src->title);
     strcpy (dest->album, src->album);
+    strcpy (dest->composed_metadata, src->composed_metadata);
     dest->save_track = src->save_track;
 }
 
+/**** The main loop for ripping ****/
 error_code
 ripstream_rip()
 {
@@ -237,24 +256,22 @@ ripstream_rip()
     u_long extract_size;
 
     /* get the data & meta-data from the stream */
-    debug_printf ("RIPSTREAM_RIP: get_stream_data:%p\n",m_in);
-    ret = m_in->get_stream_data(m_getbuffer, m_current_track.raw_metadata);
+    debug_printf ("RIPSTREAM_RIP: top of loop\n");
+    ret = get_stream_data(m_getbuffer, m_current_track.raw_metadata);
     if (ret != SR_SUCCESS) {
-	debug_printf("m_in->get_stream_data bad return code: %d\n", ret);
+	debug_printf("get_stream_data bad return code: %d\n", ret);
 	return ret;
     }
 
-    /* Immediately dump data to relay & show file */
-    rip_manager_put_raw_data (m_getbuffer, m_ripchunk_size);
-
-    /* First time through, determine the bitrate. 
+    /* First time through, need to determine the bitrate. 
        The bitrate is needed to do the track splitting parameters 
        properly in seconds.  See the readme file for details.  */
+    /* GCS FIX: For VBR streams, the header value may be more reliable. */
     if (m_bitrate == -1) {
         unsigned long test_bitrate;
 	debug_printf("Querying stream for bitrate - first time.\n");
 	if (m_content_type == CONTENT_TYPE_MP3) {
-	    find_bitrate(&test_bitrate, m_getbuffer, m_ripchunk_size);
+	    find_bitrate(&test_bitrate, m_getbuffer, m_buffersize);
 	    m_bitrate = test_bitrate / 1000;
 	    debug_printf("Got bitrate: %d\n",m_bitrate);
 	} else {
@@ -269,18 +286,34 @@ ripstream_rip()
 	    else
 		m_bitrate = 24;
 	}
-        compute_cbuf2_size (m_sp_opt, m_bitrate, m_in->getsize);
-	ret = cbuf2_init(&m_cbuffer, m_in->getsize * m_cbuf2_size);
+        compute_cbuf2_size (m_sp_opt, m_bitrate, m_buffersize);
+	ret = cbuf2_init(&g_cbuf2, m_have_relay, m_buffersize, m_cbuf2_size);
 	if (ret != SR_SUCCESS) return ret;
     }
 
-    /* Parse metadata */
-    if (m_current_track.raw_metadata[0]) {
-        parse_metadata (&m_current_track);
+    if (m_external_process) {
+	/* If using external metadata, check for that */
+	clear_track_info (&m_current_track);
+	read_external (m_external_process, &m_current_track);
     } else {
-        clear_track_info (&m_current_track);
+	/* Parse the metadata (RMK: ogg might have multiple metadata/chunk) */
+	if (m_current_track.raw_metadata[0]) {
+	    parse_metadata (&m_current_track);
+	} else {
+	    clear_track_info (&m_current_track);
+	}
     }
 
+    /* Copy the data into cbuffer */
+    ret = cbuf2_insert_chunk(&g_cbuf2, m_getbuffer, m_buffersize,
+			     &m_current_track);
+    if (ret != SR_SUCCESS) {
+	debug_printf("start_track had bad return code %d\n", ret);
+	return ret;
+    }
+
+    filelib_write_show (m_getbuffer, m_buffersize);
+
     /* First time through, so start a track. */
     if (m_first_time_through) {
 	int ret;
@@ -300,34 +333,23 @@ ripstream_rip()
 	copy_track_info (&m_old_track, &m_current_track);
     }
 
-    /* Copy the data into cbuffer */
-    ret = cbuf2_insert_chunk(&m_cbuffer, m_getbuffer, m_in->getsize);
-    if (ret != SR_SUCCESS) {
-	debug_printf("start_track had bad return code %d\n", ret);
-	return ret;
-    }
-
     /* Check for track change. */
+    debug_printf ("m_current_track.have_track_info = %d\n", m_current_track.have_track_info);
     if (m_current_track.have_track_info && is_track_changed()) {
 	/* Set m_find_silence equal to the number of additional blocks 
 	   needed until we can do silence separation. */
 	debug_printf ("VERIFIED TRACK CHANGE (m_find_silence = %d)\n",
 		      m_find_silence);
 	copy_track_info (&m_new_track, &m_current_track);
-	relaylib_send_meta_data (m_current_track.raw_metadata);
 	if (m_find_silence < 0) {
-	    if (m_mi_to_cbuf2_end > 0) {
-		m_find_silence = m_mi_to_cbuf2_end;
+	    if (m_mic_to_cb_end > 0) {
+		m_find_silence = m_mic_to_cb_end;
 	    } else {
 		m_find_silence = 0;
 	    }
 	}
-    } else {
-	if (m_meta_interval != NO_META_INTERVAL)
-	    relaylib_send_meta_data (0);
     }
 
-    debug_printf ("Checking for silence\n");
     format_track_info (&m_old_track, "old");
     format_track_info (&m_new_track, "new");
     format_track_info (&m_current_track, "current");
@@ -335,8 +357,15 @@ ripstream_rip()
     if (m_find_silence == 0) {
 	/* Find separation point */
 	u_long pos1, pos2;
+	debug_printf ("m_find_silence == 0\n");
 	ret = find_sep (&pos1, &pos2);
-	if (ret != SR_SUCCESS) {
+	if (ret == SR_ERROR_REQUIRED_WINDOW_EMPTY) {
+	    /* If this happens, the previous song should be truncated to 
+	       zero bytes. */
+	    pos1 = 0;
+	    pos2 = 0;
+	}
+	else if (ret != SR_SUCCESS) {
 	    debug_printf("find_sep had bad return code %d\n", ret);
 	    return ret;
 	}
@@ -345,7 +374,7 @@ ripstream_rip()
 	ret = end_track(pos1, pos2, &m_old_track);
 	if (ret != SR_SUCCESS)
 	    real_ret = ret;
-        m_cue_sheet_bytes += pos2;
+	m_cue_sheet_bytes += pos2;
 
 	/* Start next track */
 	ret = start_track (&m_new_track);
@@ -358,67 +387,57 @@ ripstream_rip()
     if (m_find_silence >= 0) m_find_silence --;
 
     /* If buffer almost full, dump extra to current song. */
-    if (cbuf2_get_free(&m_cbuffer) < m_in->getsize) {
-        /* Extract only as much as needed */
-	extract_size = m_in->getsize - cbuf2_get_free(&m_cbuffer);
-        ret = cbuf2_extract(&m_cbuffer, m_getbuffer, extract_size);
+    if (cbuf2_get_free(&g_cbuf2) < m_buffersize) {
+	u_long curr_song;
+	debug_printf ("cbuf2_get_free < m_buffersize\n");
+	extract_size = m_buffersize - cbuf2_get_free(&g_cbuf2);
+        ret = cbuf2_extract(&g_cbuf2, m_getbuffer, extract_size, &curr_song);
         if (ret != SR_SUCCESS) {
 	    debug_printf("cbuf2_extract had bad return code %d\n", ret);
 	    return ret;
 	}
+
 	/* Post to caller */
-	m_cue_sheet_bytes += extract_size;
-	rip_manager_put_data(m_getbuffer, extract_size);
+	if (curr_song < extract_size) {
+	    u_long curr_song_bytes = extract_size - curr_song;
+	    m_cue_sheet_bytes += curr_song_bytes;
+	    rip_manager_put_data (&m_getbuffer[curr_song], curr_song_bytes);
+	}
     }
 
     return real_ret;
 }
 
-static u_long
-clip_to_cbuffer (int pos)
-{
-    if (pos < 0) {
-	return 0;
-    } else if ((u_long)pos > cbuf2_get_used(&m_cbuffer)) {
-	return cbuf2_get_used(&m_cbuffer);
-    } else {
-	return (u_long) pos;
-    }
-}
-
-/* New version */
 error_code
 find_sep (u_long *pos1, u_long *pos2)
 {
-    int pos1i, pos2i;
-    int sw_start, sw_end, sw_sil;
-    u_long psilence;
+    int rw_start, rw_end, sw_sil;
     int ret;
 
     debug_printf ("*** Finding separation point\n");
 
     /* First, find the search region w/in cbuffer. */
-    sw_start = cbuf2_get_used(&m_cbuffer) - m_sw_start_to_cbuf2_end;
-    if (sw_start < 0) sw_start = 0;
-    sw_end = cbuf2_get_used(&m_cbuffer) - m_sw_end_to_cbuf2_end;
-    if (sw_end < 0) sw_end = 0;
-    debug_printf ("search window (bytes): %d,%d,%d\n", sw_start, sw_end,
-		  cbuf2_get_used(&m_cbuffer));
+    rw_start = g_cbuf2.item_count - m_rw_start_to_cb_end;
+    if (rw_start < 0) {
+	return SR_ERROR_REQUIRED_WINDOW_EMPTY;
+    }
+    rw_end = g_cbuf2.item_count - m_rw_end_to_cb_end;
+    if (rw_end < 0) {
+	return SR_ERROR_REQUIRED_WINDOW_EMPTY;
+    }
+
+    debug_printf ("search window (bytes): %d,%d,%d\n", rw_start, rw_end,
+		  g_cbuf2.item_count);
 
     if (m_content_type != CONTENT_TYPE_MP3) {
-        /* If the search region is too small, take the middle. */
-	sw_sil = (sw_end + sw_start) / 2;
+	sw_sil = (rw_end + rw_start) / 2;
 	debug_printf ("(not mp3) taking middle: sw_sil=%d\n", sw_sil);
-    }
-    else if (sw_end - sw_start < m_min_search_win) {
-        /* If the search region is too small, take the middle. */
-	sw_sil = (sw_end + sw_start) / 2;
-	debug_printf ("taking middle: sw_sil=%d\n", sw_sil);
+	*pos1 = rw_start + sw_sil;
+	*pos2 = rw_start + sw_sil;
     } else {
-	/* Otherwise, copy from search window into private buffer */
-	int bufsize = sw_end - sw_start;
+	int bufsize = rw_end - rw_start;
 	char* buf = (u_char *)malloc(bufsize);
-	ret = cbuf2_peek_rgn (&m_cbuffer, buf, sw_start, bufsize);
+	ret = cbuf2_peek_rgn (&g_cbuf2, buf, rw_start, bufsize);
 	if (ret != SR_SUCCESS) {
 	    debug_printf ("PEEK FAILED: %d\n", ret);
 	    free(buf);
@@ -427,29 +446,19 @@ find_sep (u_long *pos1, u_long *pos2)
 	debug_printf ("PEEK OK\n");
 
 	/* Find silence point */
-	ret = findsep_silence(buf, bufsize, m_sp_opt->xs_silence_length,
-			      &psilence);
+	/* GCS FIX: This doesn't yet consider sw vs rw */
+	ret = findsep_silence (buf, bufsize,
+			       m_rw_start_to_sw_start,
+			       m_sp_opt->xs_search_window_1 
+			       + m_sp_opt->xs_search_window_2,
+			       m_sp_opt->xs_silence_length,
+			       m_sp_opt->xs_padding_1,
+			       m_sp_opt->xs_padding_2,
+			       pos1, pos2);
+	*pos1 += rw_start;
+	*pos2 += rw_start;
 	free(buf);
-	if (ret != SR_SUCCESS && ret != SR_ERROR_CANT_DECODE_MP3) {
-	    return ret;
-	}
-	sw_sil = sw_start + psilence;
-
-	/* Add 1/2 of the silence window to get middle */
-	sw_sil = sw_sil + ms_to_bytes(m_sp_opt->xs_silence_length/2,m_bitrate);
     }
-
-    /* Compute padding.  We need pos1 == end of 1st song and 
-       pos2 == beginning of 2nd song. */
-    pos1i = sw_sil + ms_to_bytes(m_sp_opt->xs_padding_2,m_bitrate);
-    pos2i = sw_sil - ms_to_bytes(m_sp_opt->xs_padding_1,m_bitrate);
-
-    /* Clip padding to amount available in cbuffer (there could be 
-       less than the full cbuffer in a few cases, such as track 
-       changes close to each other). */
-    *pos1 = clip_to_cbuffer (pos1i);
-    *pos2 = clip_to_cbuffer (pos2i);
-
     return SR_SUCCESS;
 }
 
@@ -466,28 +475,21 @@ end_track(u_long pos1, u_long pos2, TRACK_INFO* ti)
 
     // pos1 is end of prev track
     // pos2 is beginning of next track
+    // positions are relative to cbuf2->read_index
 
     // First, dump the part only in prev track
-    if (pos1 <= pos2) {
-	if ((ret = cbuf2_extract(&m_cbuffer, buf, pos1)) != SR_SUCCESS)
-	    goto BAIL;
-	// Next, skip past portion not in either track
-	if (pos1 < pos2) {
-	    if ((ret = cbuf2_fastforward(&m_cbuffer, pos2-pos1)) != SR_SUCCESS)
-		goto BAIL;
-	}
-    } else {
-	if ((ret = cbuf2_extract(&m_cbuffer, buf, pos2)) != SR_SUCCESS)
-	    goto BAIL;
-	// Next, grab, but don't skip, past portion in both tracks
-	if ((ret = cbuf2_peek(&m_cbuffer, buf+pos2, pos1-pos2)) != SR_SUCCESS)
-	    goto BAIL;
-    }
+    ret = cbuf2_peek(&g_cbuf2, buf, pos1);
+    if (ret != SR_SUCCESS) goto BAIL;
+
+    // Let cbuf know about the start of the next track
+    cbuf2_set_next_song (&g_cbuf2, pos2);
 
     // Write that out to the current file
+    // GCS FIX: m_bytes_ripped is incorrect when there is padding
     if ((ret = rip_manager_put_data(buf, pos1)) != SR_SUCCESS)
 	goto BAIL;
 
+    /* This is id3v1 */
     if (m_addID3tag) {
 	ID3Tag id3;
 	memset(&id3, '\000',sizeof(id3));
@@ -496,8 +498,10 @@ end_track(u_long pos1, u_long pos2, TRACK_INFO* ti)
 	strncpy(id3.songtitle, ti->title, sizeof(id3.songtitle));
 	strncpy(id3.album, ti->album, sizeof(id3.album));
 	id3.genre = (char) 0xFF; // see http://www.id3.org/id3v2.3.0.html#secA
-	if ((ret = rip_manager_put_data((char *)&id3, sizeof(id3))) != SR_SUCCESS)
+	ret = rip_manager_put_data((char *)&id3, sizeof(id3));
+	if (ret != SR_SUCCESS) {
 	    goto BAIL;
+	}
     }
 
     // Only save this track if we've skipped over enough cruft 
@@ -640,7 +644,7 @@ ms_to_blocks (int ms, int bitrate, int round_up)
     int ms_abs = ms > 0 ? ms : -ms;
     int ms_sign = ms > 0 ? 1 : 0;
     int bits = ms_abs * bitrate;
-    int bits_per_block = 8 * m_ripchunk_size;
+    int bits_per_block = 8 * m_buffersize;
     int blocks = bits / bits_per_block;
     if (bits % bits_per_block > 0) {
 	if (!(round_up ^ ms_sign)) {
@@ -677,7 +681,13 @@ static void
 compute_cbuf2_size (SPLITPOINT_OPTIONS *sp_opt, int bitrate, 
 		      int meta_interval)
 {
-    u_long sws;
+    long sws, sl;
+    long mic_to_mi;
+    long prepad, postpad;
+    long offset;
+    long mic_to_sw_start, mic_to_sw_end;
+    long mic_to_rw_start, mic_to_rw_end;
+    long mic_to_cb_start, mic_to_cb_end;
 
     debug_printf ("---------------------------------------------------\n");
     debug_printf ("xs_search_window: %d,%d\n",
@@ -687,87 +697,190 @@ compute_cbuf2_size (SPLITPOINT_OPTIONS *sp_opt, int bitrate,
 		  sp_opt->xs_padding_2);
     debug_printf ("xs_offset: %d\n", sp_opt->xs_offset);
     debug_printf ("---------------------------------------------------\n");
-    
-    /* compute the search window size */
-    sws = sp_opt->xs_search_window_1 + sp_opt->xs_search_window_2;
-
-    /* compute interval from mi to beginning of buffer */
-    m_mi_to_cbuf2_start = sp_opt->xs_offset - sp_opt->xs_search_window_1
-	+ sp_opt->xs_silence_length/2 - sp_opt->xs_padding_1;
-    debug_printf ("m_mi_to_cbuf2_start (ms): %d\n", m_mi_to_cbuf2_start);
-
-    /* compute interval from mi to end of buffer */
-    m_mi_to_cbuf2_end = sp_opt->xs_offset + sp_opt->xs_search_window_2
-	- sp_opt->xs_silence_length/2 + sp_opt->xs_padding_2;
-    debug_printf ("m_mi_to_cbuf2_end (ms): %d\n", m_mi_to_cbuf2_end);
     debug_printf ("bitrate = %d, meta_inf = %d\n", bitrate, meta_interval);
+    debug_printf ("---------------------------------------------------\n");
+    
+    /* mic_to_mi is the "half of a meta-inf" from the meta inf 
+       change to the previous (non-changed) meta inf */
+    mic_to_mi = meta_interval / 2;
+    debug_printf ("mic_to_mi: %d\n", mic_to_mi);
 
-    debug_printf ("CBUF2 (ms): %d:%d\n",m_mi_to_cbuf2_start,
-	m_mi_to_cbuf2_end);
+    /* compute the search window size (sws) */
+    sws = ms_to_bytes (sp_opt->xs_search_window_1, bitrate) 
+	    + ms_to_bytes (sp_opt->xs_search_window_2, bitrate);
+    debug_printf ("sws: %d\n", sws);
 
-    /* convert from ms to meta-inf blocks */
-    m_mi_to_cbuf2_start = ms_to_blocks (m_mi_to_cbuf2_start, bitrate, 0);
-    m_mi_to_cbuf2_end = ms_to_blocks (m_mi_to_cbuf2_end, bitrate, 1);
+    /* compute the silence length (sl) */
+    sl = ms_to_bytes (sp_opt->xs_silence_length, bitrate);
+    debug_printf ("sl: %d\n", sl);
 
-    /* Case 1 -- see readme (GCS) */
-    if (m_mi_to_cbuf2_start > 0) {
-	debug_printf ("CASE 1\n");
-        m_cbuf2_size = m_mi_to_cbuf2_end;
-	m_sw_end_to_cbuf2_end = sp_opt->xs_padding_2 
-	    - sp_opt->xs_silence_length/2;
-	/* GCS: Mar 27, 2004 - it's possible for padding to be less 
-	   than search window */
-	if (m_sw_end_to_cbuf2_end < 0) {
-	    m_sw_end_to_cbuf2_end = 0;
-	}
-	m_sw_start_to_cbuf2_end = m_sw_end_to_cbuf2_end + sws;
+    /* compute padding */
+    prepad = ms_to_bytes (sp_opt->xs_padding_1, bitrate);
+    postpad = ms_to_bytes (sp_opt->xs_padding_2, bitrate);
+    debug_printf ("padding: %d %d\n", prepad, postpad);
+
+    /* compute offset */
+    offset = ms_to_bytes(sp_opt->xs_offset,bitrate);
+    debug_printf ("offset: %d\n", offset);
+
+    /* compute interval from mi to search window */
+    mic_to_sw_start = - mic_to_mi + offset 
+	    - ms_to_bytes(sp_opt->xs_search_window_1,bitrate);
+    mic_to_sw_end = - mic_to_mi + offset 
+	    + ms_to_bytes(sp_opt->xs_search_window_2,bitrate);
+    debug_printf ("mic_to_sw_start: %d\n", mic_to_sw_start);
+    debug_printf ("mic_to_sw_end: %d\n", mic_to_sw_end);
+
+    /* compute interval from mi to required window */
+    mic_to_rw_start = mic_to_sw_start + sl / 2 - prepad;
+    if (mic_to_rw_start > mic_to_sw_start) {
+	mic_to_rw_start = mic_to_sw_start;
     }
-
-    /* Case 2 */
-    else if (m_mi_to_cbuf2_end >= 0) {
-	debug_printf ("CASE 2\n");
-        m_cbuf2_size = m_mi_to_cbuf2_end - m_mi_to_cbuf2_start;
-	m_sw_end_to_cbuf2_end = sp_opt->xs_padding_2 
-	    - sp_opt->xs_silence_length/2;
-	/* GCS: Mar 27, 2004 - it's possible for padding to be less 
-	   than search window */
-	if (m_sw_end_to_cbuf2_end < 0) {
-	    m_sw_end_to_cbuf2_end = 0;
-	}
-	m_sw_start_to_cbuf2_end = m_sw_end_to_cbuf2_end + sws;
+    mic_to_rw_end = mic_to_sw_end - sl / 2 + postpad;
+    if (mic_to_rw_end < mic_to_sw_end) {
+	mic_to_rw_end = mic_to_sw_end;
     }
+    debug_printf ("mic_to_rw_start: %d\n", mic_to_rw_start);
+    debug_printf ("mic_to_rw_end: %d\n", mic_to_rw_end);
 
-    /* Case 3 */
-    else {
-	debug_printf ("CASE 3\n");
-        m_cbuf2_size = - m_mi_to_cbuf2_start;
-        m_sw_start_to_cbuf2_end = m_cbuf2_size 
-	    + sp_opt->xs_silence_length/2 
-	    - sp_opt->xs_padding_1;
-	/* GCS: Mar 27, 2004 - it's possible for padding to be less 
-	   than search window */
-	if (m_sw_start_to_cbuf2_end > m_cbuf2_size) {
-	    m_sw_start_to_cbuf2_end = m_cbuf2_size;
-	}
-	m_sw_end_to_cbuf2_end = m_sw_start_to_cbuf2_end + sws;
+    /* This code replaces the 3 cases (see OBSOLETE in gcs_notes.txt) */
+    mic_to_cb_start = mic_to_rw_start;
+    mic_to_cb_end = mic_to_rw_end;
+    if (mic_to_cb_start > -meta_interval) {
+	mic_to_cb_start = -meta_interval;
     }
+    if (mic_to_cb_end < 0) {
+	mic_to_cb_end = 0;
+    }
+    debug_printf ("mic_to_cb_start: %d\n", mic_to_cb_start);
+    debug_printf ("mic_to_cb_end: %d\n", mic_to_cb_end);
 
-    /* Convert these from ms to bytes */
-    debug_printf ("SEARCH WIN (ms): %d | %d | %d\n",
-		  m_sw_start_to_cbuf2_end,
-		  sws, m_sw_end_to_cbuf2_end);
-    m_sw_start_to_cbuf2_end = ms_to_bytes (m_sw_start_to_cbuf2_end, 
-	bitrate);
-    m_sw_end_to_cbuf2_end = ms_to_bytes (m_sw_end_to_cbuf2_end, bitrate);
+    /* Convert to chunks & compute cbuf size */
+    mic_to_cb_end = (mic_to_cb_end + (meta_interval-1)) / meta_interval;
+    mic_to_cb_start = -((-mic_to_cb_start + (meta_interval-1)) 
+			/ meta_interval);
+    m_cbuf2_size = -mic_to_cb_start + mic_to_cb_end;
+    if (m_cbuf2_size < 3) {
+	m_cbuf2_size = 3;
+    }
+    debug_printf ("CBUF2 (BLOCKS): %d:%d -> %d\n", mic_to_cb_start,
+		  mic_to_cb_end, m_cbuf2_size);
 
-    /* Get minimum search window (in bytes) */
-    m_min_search_win = sp_opt->xs_silence_length + (sws - sp_opt->xs_silence_length) / 2 ;
-    m_min_search_win = ms_to_bytes (m_min_search_win, bitrate);
-
-    debug_printf ("CBUF2 (BLOCKS): %d:%d -> %d\n",m_mi_to_cbuf2_start,
-	m_mi_to_cbuf2_end, m_cbuf2_size);
-    debug_printf ("SEARCH WIN (bytes): %d | %d | %d\n",
-		  m_sw_start_to_cbuf2_end,
-		  ms_to_bytes(sws,bitrate),
-		  m_sw_end_to_cbuf2_end);
+    /* Set some global variables to be used by splitting algorithm */
+    m_mic_to_cb_end = mic_to_cb_end;
+    m_rw_start_to_cb_end = mic_to_cb_end * meta_interval - mic_to_rw_start;
+    m_rw_end_to_cb_end = mic_to_cb_end * meta_interval - mic_to_rw_end;
+    m_rw_start_to_sw_start = sp_opt->xs_padding_1 
+	    - (sp_opt->xs_search_window_1 + sp_opt->xs_search_window_2);
+    if (m_rw_start_to_sw_start < 0) {
+	m_rw_start_to_sw_start = 0;
+    }
+    debug_printf ("m_mic_to_cb_end: %d\n", m_mic_to_cb_end);
+    debug_printf ("m_rw_start_to_cb_end: %d\n", m_rw_start_to_cb_end);
+    debug_printf ("m_rw_end_to_cb_end: %d\n", m_rw_end_to_cb_end);
+    debug_printf ("m_rw_start_to_sw_start: %d\n", m_rw_start_to_sw_start);
+}
+
+/* GCS: This used to be myrecv in rip_manager.c */
+static int
+ripstream_recvall (char* buffer, int size)
+{
+    int ret;
+    /* GCS: Jun 5, 2004.  Here is where I think we are getting aussie's 
+       problem with the SR_ERROR_INVALID_METADATA or SR_ERROR_NO_TRACK_INFO
+       messages */
+    ret = socklib_recvall(&m_sock, buffer, size, m_timeout);
+    if (ret >= 0 && ret != size) {
+	debug_printf ("rip_manager_recv: expected %d, got %d\n",size,ret);
+	ret = SR_ERROR_RECV_FAILED;
+    }
+    return ret;
+}
+
+static error_code
+get_stream_data (char *data_buf, char *track_buf)
+{
+    int ret = 0;
+    char c;
+    char newtrack[MAX_TRACK_LEN];
+
+    *track_buf = 0;
+    m_chunkcount++;
+    if ((ret = ripstream_recvall (data_buf, m_buffersize)) <= 0)
+	return ret;
+
+    if (m_meta_interval == NO_META_INTERVAL) {
+	return SR_SUCCESS;
+    }
+
+    if ((ret = ripstream_recvall (&c, 1)) <= 0)
+	return ret;
+
+    debug_printf ("METADATA LEN: %d\n",(int)c);
+    if (c < 0) {
+	debug_printf ("Got invalid metadata: %d\n",c);
+	return SR_ERROR_INVALID_METADATA;
+    } else if (c == 0) {
+	// around christmas time 2001 someone noticed that 1.8.7 shoutcast
+	// none-meta servers now just send a '0' if they have no meta data
+	// anyway, bassicly if the first meta capture is null then we assume
+	// that the stream does not have metadata
+	return SR_SUCCESS;
+    } else {
+	if ((ret = get_track_from_metadata (c * 16, newtrack)) != SR_SUCCESS) {
+	    debug_printf("get_trackname had a bad return %d", ret);
+	    return ret;
+	}
+
+	// WolfFM is a station that does not stream meta-data, but is in that format anyway...
+	// StreamTitle='', so we need to pretend this has no meta data.
+	if (*newtrack == '\0') {
+	    return SR_SUCCESS;
+	}
+	/* This is the case where we got metadata. */
+	strncpy(track_buf, newtrack, MAX_TRACK_LEN);
+    }
+    return SR_SUCCESS;
+}
+
+static error_code
+get_track_from_metadata (int size, char *newtrack)
+{
+    int i;
+    int ret;
+    char *namebuf;
+
+    if ((namebuf = malloc(size)) == NULL)
+	return SR_ERROR_CANT_ALLOC_MEMORY;
+
+    if ((ret = ripstream_recvall (namebuf, size)) <= 0) {
+	free(namebuf);
+	return ret;
+    }
+
+    debug_printf ("METADATA TITLE\n");
+    for (i=0; i<size; i++) {
+	debug_printf ("%2x ",(unsigned char)namebuf[i]);
+	if (i % 20 == 19) {
+	    debug_printf ("\n");
+	}
+    }
+    debug_printf ("\n");
+    for (i=0; i<size; i++) {
+	debug_printf ("%2c ",namebuf[i]);
+	if (i % 20 == 19) {
+	    debug_printf ("\n");
+	}
+    }
+    debug_printf ("\n");
+
+    if(strstr(namebuf, "StreamTitle='") == NULL) {
+	free(namebuf);
+	return SR_ERROR_NO_TRACK_INFO;
+    }
+    subnstr_until(namebuf+strlen("StreamTitle='"), "';", newtrack, MAX_TRACK_LEN);
+    trim(newtrack);
+
+    free(namebuf);
+    return SR_SUCCESS;
 }
